var helpers = require('./helpers'),
    controllers = module.exports,
    globals = {},
    models;

var addHelpers = function(req, res) {
    res.locals.capitalizeFirstLetter = helpers.capitalizeFirstLetter;
    res.locals.staticUrl = '/public' + globals.url;
    res.locals.base = globals.url;
    res.locals.menu = globals.paths;
    res.locals['mapData'] = function(object, ref, token) {
        if (ref.indexOf('.') != -1) {
            var parts = ref.split(token);
            var get = function(context, variable, rest) {
                if (!context) {
                    return "";
                }
                if (context || context[variable]) {
                    if (rest.length > 0) {
                        return get(context[variable], _.first(rest), _.rest(rest)) || "";
                    } else {
                        var result = context[variable];
                        return (_.isFunction(result) ? result.call(null) : result) || "";
                    }
                } else {
                    return "";
                }
            }
            object[ref] = get(object, _.first(parts), _.rest(parts));
        }
        return "";
    };
    res.locals['mapName'] = function(ref, token) {
        if (ref.indexOf('.') != -1) {
            var parts = ref.split('.');
            var name = parts[0];
            for (var i = 1; i < parts.length; i++) {
                name += '[' + parts[i] + ']';
            }
            return name;
        }
        return ref;
    }
    res.locals['isType'] = function(param, typeString) {
        return typeof(param) == typeString;
    };
    res.locals['string'] = function(param) {
        return param.toString();
    };
    res.locals['log'] = function(input) {
        console.log('INPUT:', input);
        return '';
    };
    res.locals['exists'] = function(value, values) {
        if (typeof value == typeof values) {
            return value == values;
        } else {
            if (values instanceof Array) {
                var _ = require('underscore');
                return _.find(values, function(item) {
                    if (({}).toString.call(item).match(/\s([a-zA-Z]+)/)[1].toLowerCase() == "object") {
                        return item.id == value;
                    } else {
                        return item == value;
                    }
                });
            } else {
                if (value == values.id) {
                    return value == values.id;
                } else {
                    return value == values.toString();
                }
            }
        }
    }
    res.locals['decrease'] = function(value) {
        return value - 1;
    }
    res.locals['increase'] = function(value) {
        return value + 1;
    }
    res.locals['request'] = req;
}

var renderTemplate = function(filename, args, res) {
    var swig = require('swig');
    for (var i in res.locals) {
        args[i] = res.locals[i];
    }
    swig.setDefaults({ cache: false });
    var template = swig.compileFile(__dirname + '/../templates/' + filename + '.html');
    if (globals.templates) {
        template = swig.compileFile(globals.templates + '/' + filename + '.html');
    }
    return res.send(template(args));
};

controllers.setGlobals = function(opts) {
    globals = opts;
    helpers.setGlobals(globals);
    models = require('./models.js')(globals.mongoose);
};

controllers.index = function(req, res) {
    addHelpers(req, res);
    if (req.method == "post" || req.method == "POST") {
        var Admin = models.Admin;
        Admin.find({
            email: req.body.email,
            password: helpers.sha1(req.body.password),
            active: true
        }, function(err, model) {
            if (err) renderTemplate('500', {}, res);
            if (model == null) {
                renderTemplate('login', {
                    error: 'Access denied'
                }, res);
            } else {
                res.cookie(globals.secret, model._id, { maxAge: 315532800000 });
                renderTemplate('index', {}, res);
            }
        });
    } else {
        if (req.cookies[globals.secret]) {
            renderTemplate('index', {}, res);
        } else {
            delete req.cookies[globals.secret];
            renderTemplate('login', {}, res);
        }
    }
};

// List completed
controllers.list = function(req, res) {
    addHelpers(req, res);
    if (req.cookies[globals.secret]) {
        var p = req.params.path,
            info = globals.info[p],
            Model = globals.mongoose.model(info.model);

        var page = (req.param('page') || 1) - 1;
        var perPage = req.param('perpage') || 30;
        var options = {
            perPage: perPage,
            page: page,
            meta: info
        };

        Model.list(options, function(err, results) {
            if (err) return renderTemplate('500', {}, res);
            Model.count().exec(function(err, count) {
                renderTemplate('list', {
                    title: helpers.capitalizeFirstLetter(info.label),
                    list: info.list || [],
                    data: results,
                    path: p,
                    page: page + 1,
                    perPage: perPage,
                    pages: Math.ceil(count / perPage)
                }, res);
            });
        });
    } else {
        res.redirect(globals.url);
    }
};

controllers.save = function(req, res) {
    addHelpers(req, res);
    if (req.cookies[globals.secret]) {
        helpers.setGlobals(globals);
        var id = req.params.id,
            p = req.params.path,
            Model = globals.mongoose.model(globals.info[p].model);
        Model.findOne({
            _id: id
        }, function(err, doc) {
            if (err) console.error(err);
            helpers.processFormFields(globals.info[p], req.body, function() {
                if (!id) {
                    doc = new Model(req.body);
                    doc.password = '123change';
                } else {
                    helpers.parseJSON(req.body);
                    helpers.updateFromObject(doc, req.body);
                }
                doc.save(function(err, result) {
                    if (err) console.error(err);
                    return res.redirect(globals.url + '/' + p);
                });
            });
        });
    } else {
        res.redirect(globals.url);
    }
};

controllers.view = function(req, res) {
    addHelpers(req, res);
    if (req.cookies[globals.secret]) {
        helpers.setGlobals(globals);
        var p = req.params.path,
            id = req.params.id,
            meta = globals.info[p],
            Model = globals.mongoose.model(meta.model);

        var options = {
            meta: meta
        };

        Model.details(id, options, function(err, doc) {
            if (err) return res.redirect('404');
            helpers.processEditFields(meta, doc, function() {
                renderTemplate('view', {
                    meta: meta,
                    doc: doc,
                    path: p,
                    edit: meta.edit,
                    fields: meta.fields
                }, res);
            });
        });
    } else {
        res.redirect('/admin');
    }
};

controllers.edit = function(req, res) {
    addHelpers(req, res);
    if (req.cookies[globals.secret]) {
        var p = req.params.path,
            id = req.params.id,
            info = globals.info[p],
            Model = globals.mongoose.models[info.model];
        Model.load(id, function(err, doc) {
            if (err) return renderTemplate('500', {
                error: err
            }, res);
            if (!doc) {
                doc = new Model({});
                var schema = Model.schema.tree;
                var fields = [];
                for(var schemaAttr in schema) {
                    if (doc[schemaAttr] == undefined && schemaAttr != '__v') {
                        doc[schemaAttr] = '';
                    }
                }
            }
            helpers.processEditFields(info, doc, function() {
                var docObject = JSON.parse(JSON.stringify(doc));
                docObject.isNew = doc.isNew;
                renderTemplate('edit', {
                    meta: info,
                    doc: docObject,
                    path: p,
                    edit: info.edit,
                    fields: info.fields
                }, res);
            });
        });
    } else {
        res.redirect('/admin');
    }
};

controllers.del = function(req, res) {
    addHelpers(req, res);
    if (req.cookies[globals.secret]) {
        var id = req.params.id,
            p = req.params.path,
            Model = globals.mongoose.model(globals.info[p].model);

        Model.findById(id).exec(function(err, model) {
            if (err) console.error(err);
            if (globals.info[p].destroy) {
                model.remove(function() {
                    return res.redirect(globals.url + '/' + p);
                });
            } else {
                model.active = false;
                model.save(function(err) {
                    if (err) console.error(err);
                    return res.redirect(globals.url + '/' + p);
                });
            }
        });
    } else {
        res.redirect(globals.url);
    }
};

controllers.logout = function(req, res) {
    addHelpers(req, res);
    res.clearCookie(globals.secret);
    res.redirect(globals.url);
};
